/******************************************************************************
 * menuman.hpp
 * Drop-down menu manager.
 *-----------------------------------------------------------------------------
 * Copyright (C) 2015 Osman Celimli
 * For conditions of distribution and use, see copyright notice in playtone.cpp
 ******************************************************************************/
#ifndef MENUMAN
#define MENUMAN

/******************************************************************************
 * Menu Layout
 ******************************************************************************/
/* FILE \
 *  - OPEN SAMPLES
 *  - OPEN INSTRUMENTS
 * ------------------------
 *  - OPEN MUSIC
 *  - OPEN SOUND EFFECT
 * ------------------------
 *  - LOG OUTPUT
 *  - STOP LOGGING
 * ------------------------
 *  - EXIT
 */
#define MFILE_SLOT						0
#define MFILE_OPEN_SAMPLES_SUBSLOT		0
#define MFILE_OPEN_INSTR_SUBSLOT		1
#define MFILE_OPEN_MUSIC_SUBSLOT		3
#define MFILE_OPEN_SFX_SUBSLOT			4
#define MFILE_START_LOG_SUBSLOT			6
#define MFILE_STOP_LOG_SUBSLOT			7

/* PLAYBACK \
 *  - PAUSE + MUTE
 *  - RESUME
 * ------------------------
 *  - STOP ALL
 * ------------------------
 *  - PLAY MUSIC
 *  - STOP MUSIC
 * ------------------------
 *  - PLAY SOUND EFFECT
 *  - STOP SOUND EFFECT
 */
#define MPLAYBACK_SLOT					1
#define MPLAYBACK_PAUSE_SUBSLOT			0
#define MPLAYBACK_RESUME_SUBSLOT		1
#define MPLAYBACK_STOPALL_SUBSLOT		3
#define MPLAYBACK_PLAYMUS_SUBSLOT		5
#define MPLAYBACK_STOPMUS_SUBSLOT		6
#define MPLAYBACK_PLAYSFX_SUBSLOT		8
#define MPLAYBACK_STOPSFX_SUBSLOT		9

/******************************************************************************
 * MenuMan Class
 ******************************************************************************/
#define MENUMAN_BUFFER_ENTRIES			2048
#define MENUMAN_BUFFER_LEN				(sizeof(mszFileName) / sizeof(mszFileName[0]))

typedef enum FileType_e {
	eFILETYPE_SMP = 0,
	eFILETYPE_INS,
	eFILETYPE_MUS,
	eFILETYPE_SFX,
	eFILETYPE_ALL,

	eFILETYPE_FOOTER
} FileType_t;

class MenuMan {
	public:
		MenuMan(HWND hParent, ToneMan *pToneMan);
		~MenuMan(void);

		LRESULT CmdProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
		void RefreshAll(void);

	private:
		BOOL GetFile(FileType_t eMode);
		BOOL SetupLog(void);

		TCHAR mszBuffer[MENUMAN_BUFFER_ENTRIES];
		HWND mhParent;
		ToneMan *mpToneMan;

		HMENU mhMain;
		HMENU mhSub_File;
		HMENU mhSub_Play;

		OPENFILENAME mOpenParams,mSaveParams;
		TCHAR mszFileName[MENUMAN_BUFFER_ENTRIES];
		TCHAR mszSaveName[MENUMAN_BUFFER_ENTRIES];
};

#endif
