/******************************************************************************
 * statman.cpp
 * Driver status visualization and additional playback control manager.
 *-----------------------------------------------------------------------------
 * Copyright (C) 2015 Osman Celimli
 * For conditions of distribution and use, see copyright notice in playtone.cpp
 ******************************************************************************/
#include <Windows.h>
#include <Commctrl.h>
#include <tchar.h>

#include <stdio.h>
#include <stdint.h>
#include <math.h>

#include "../types.h"
#include "../CoreTone/coretone.h"
#include "../WinTone/wintone.hpp"
#include "toneman.hpp"
#include "statman.hpp"
#include "resource.h"

/* VS6's Commctrl doesn't provide a #def for "STATIC" window classes,
 * this was added as "WC_STATIC" in later versions.
 */
#ifndef WC_STATIC
	#define WC_STATIC					_T("STATIC")
#endif

#ifndef WC_BUTTON
	#define WC_BUTTON					_T("BUTTON")
#endif

/******************************************************************************
 * !!!!----                      LABEL TABEL                         ----!!!!
 ******************************************************************************/
const TCHAR szName_ctInfoB[] = _T("CoreTone Info Border");
const TCHAR szName_ctInfoL[] = _T("CoreTone Info");

const TCHAR szData_ctInfoS[] = _T("Macro Channels : %u\n"
								  "Render && Decode Rates : %uHz, %uHz\n"
								  "Sample Library : %u Entries, %u Samples Each (Max)\n"
								  "Music Mood : %d                           \n"
								  "[%s] Render Ticks : %u, %.2lf sec                           ");
const TCHAR szData_ctInfoI[] = _T("Idle");
const TCHAR szData_ctInfoR[] = _T("Running");


const TCHAR szName_waveVuB[] = _T("Wave View Border");
const TCHAR szName_waveVuL[] = _T("Wave View");


const TCHAR szName_musVolT[] = _T("Music Volume Trackbar");
const TCHAR szName_musVolB[] = _T("Music Volume Border");
const TCHAR szName_musVolL[] = _T("Music Volume");

const TCHAR szLabel_musVolSoft[] = _T("Soft");
const TCHAR szLabel_musVolLoud[] = _T("Loud");


const TCHAR szName_sfxSelB[] = _T("Sound Effect Manager Border");
const TCHAR szName_sfxSelL[] = _T("Sound Effect Control");
const TCHAR szName_sfxSelP[] = _T("Sound Effect Action (Play)");
const TCHAR szName_sfxSelS[] = _T("Sound Effect Action (Stop)");

const TCHAR szName_sfxPriT[] = _T("Sound Effect Priority Trackbar");
const TCHAR szName_sfxPanT[] = _T("Sound Effect Panning Trackbar");
const TCHAR szLabel_sfxPri[] = _T("Low            High");
const TCHAR szLabel_sfxPan[] = _T("Left            Right");
const TCHAR szCat_sfxPri[] = _T("PRI");
const TCHAR szCat_sfxPan[] = _T("PAN");


const TCHAR szName_ctEmpty[] = _T("");


/******************************************************************************
 * !!!!----               CONSTRUCTOR AND DESTRUCTOR                 ----!!!!
 ******************************************************************************/
StatMan::StatMan(HWND hParent, ToneMan *pToneMan)
{
	mhParent = hParent;
	mpToneMan = pToneMan;

	/**
	 * ---- CORETONE INFO ----
	 */
	midBorder_ctInfo = WSTAT_CTINFOB_ID;
	mhBorder_ctInfo = CreateWindowEx(
		(0),									/* dwExStyle */
		WC_STATIC,								/* lpClassName */
		szName_ctInfoB,							/* lpWindowName */
		(WS_CHILD | WS_VISIBLE)
			| (SS_ETCHEDFRAME | SS_LEFT),		/* dwStyle */
		WSTAT_CTINFOB_X,						/* x */
		WSTAT_CTINFOB_Y,						/* y */
		WSTAT_CTINFOB_WIDTH,					/* nWidth */
		WSTAT_CTINFOB_HEIGHT,					/* nHeight */
		mhParent,								/* hWndParent */
		(HMENU)midBorder_ctInfo,				/* idControl */
		GetModuleHandle(NULL),					/* hInstance */
		NULL);									/* lpParam */
	SetWindowText(mhBorder_ctInfo, szName_ctEmpty);

	midLabel_ctInfo = WSTAT_CTINFOL_ID;
	mhLabel_ctInfo = CreateWindowEx(
		(0),									/* dwExStyle */
		WC_STATIC,								/* lpClassName */
		szName_ctInfoL,							/* lpWindowName */
		(WS_CHILD | WS_VISIBLE)
			| (SS_LEFT),						/* dwStyle */
		WSTAT_CTINFOL_X,						/* x */
		WSTAT_CTINFOL_Y,						/* y */
		WSTAT_CTINFOL_WIDTH,					/* nWidth */
		WSTAT_CTINFOL_HEIGHT,					/* nHeight */
		mhParent,								/* hWndParent */
		(HMENU)midLabel_ctInfo,					/* idControl */
		GetModuleHandle(NULL),					/* hInstance */
		NULL);									/* lpParam */


	/**
	 * ---- WAVE OUTPUT ----
	 */
	mhDCm = NULL;
	mhWaveMap = NULL;
	midBorder_waveVu = WSTAT_WAVEVUB_ID;
	mhBorder_waveVu = CreateWindowEx(
		(0),									/* dwExStyle */
		WC_STATIC,								/* lpClassName */
		szName_waveVuB,							/* lpWindowName */
		(WS_CHILD | WS_VISIBLE)
			| (SS_ETCHEDFRAME | SS_LEFT),		/* dwStyle */
		WSTAT_WAVEVUB_X,						/* x */
		WSTAT_WAVEVUB_Y,						/* y */
		WSTAT_WAVEVUB_WIDTH,					/* nWidth */
		WSTAT_WAVEVUB_HEIGHT,					/* nHeight */
		mhParent,								/* hWndParent */
		(HMENU)midBorder_waveVu,				/* idControl */
		GetModuleHandle(NULL),					/* hInstance */
		NULL);									/* lpParam */
	SetWindowText(mhBorder_waveVu, szName_ctEmpty);

	midLabel_waveVu = WSTAT_MUSVOLL_ID;
	mhLabel_waveVu = CreateWindowEx(
		(0),									/* dwExStyle */
		WC_STATIC,								/* lpClassName */
		szName_waveVuL,							/* lpWindowName */
		(WS_CHILD | WS_VISIBLE)
			| (SS_LEFT),						/* dwStyle */
		WSTAT_WAVEVUL_X,						/* x */
		WSTAT_WAVEVUL_Y,						/* y */
		WSTAT_WAVEVUL_WIDTH,					/* nWidth */
		WSTAT_WAVEVUL_HEIGHT,					/* nHeight */
		mhParent,								/* hWndParent */
		(HMENU)midLabel_waveVu,					/* idControl */
		GetModuleHandle(NULL),					/* hInstance */
		NULL);									/* lpParam */


	/**
	 * ---- MUSIC VOLUME TRACKBAR ----
	 */
	midBorder_musVol = WSTAT_MUSVOLB_ID;
	mhBorder_musVol = CreateWindowEx(
		(0),									/* dwExStyle */
		WC_STATIC,								/* lpClassName */
		szName_musVolB,							/* lpWindowName */
		(WS_CHILD | WS_VISIBLE)
			| (SS_ETCHEDFRAME | SS_LEFT),		/* dwStyle */
		WSTAT_MUSVOLB_X,						/* x */
		WSTAT_MUSVOLB_Y,						/* y */
		WSTAT_MUSVOLB_WIDTH,					/* nWidth */
		WSTAT_MUSVOLB_HEIGHT,					/* nHeight */
		mhParent,								/* hWndParent */
		(HMENU)midBorder_musVol,				/* idControl */
		GetModuleHandle(NULL),					/* hInstance */
		NULL);									/* lpParam */
	SetWindowText(mhBorder_musVol, szName_ctEmpty);

	midLabel_musVol = WSTAT_MUSVOLL_ID;
	mhLabel_musVol = CreateWindowEx(
		(0),									/* dwExStyle */
		WC_STATIC,								/* lpClassName */
		szName_musVolL,							/* lpWindowName */
		(WS_CHILD | WS_VISIBLE)
			| (SS_LEFT),						/* dwStyle */
		WSTAT_MUSVOLL_X,						/* x */
		WSTAT_MUSVOLL_Y,						/* y */
		WSTAT_MUSVOLL_WIDTH,					/* nWidth */
		WSTAT_MUSVOLL_HEIGHT,					/* nHeight */
		mhParent,								/* hWndParent */
		(HMENU)midLabel_musVol,					/* idControl */
		GetModuleHandle(NULL),					/* hInstance */
		NULL);									/* lpParam */

	midTrack_musVol = WSTAT_MUSVOL_ID;
	mhTrack_musVol = CreateWindowEx(
		(0),									/* dwExStyle */
		TRACKBAR_CLASS,							/* lpClassName */
		szName_musVolT,							/* lpWindowName */
		(WS_CHILD | WS_VISIBLE)
			| (TBS_AUTOTICKS),					/* dwStyle */
		WSTAT_MUSVOL_X,							/* x */
		WSTAT_MUSVOL_Y,							/* y */
		WSTAT_MUSVOL_WIDTH,						/* nWidth */
		WSTAT_MUSVOL_HEIGHT,					/* nHeight */
		mhParent,								/* hWndParent */
		(HMENU)midTrack_musVol,					/* idControl */
		GetModuleHandle(NULL),					/* hInstance */
		NULL);									/* lpParam */
	SendMessage(mhTrack_musVol, TBM_SETRANGE,
		(WPARAM)TRUE,
		(LPARAM)MAKELONG(WSTAT_MUSVOL_MIN, WSTAT_MUSVOL_MAX));
	SendMessage(mhTrack_musVol, TBM_SETPAGESIZE,
		0,
		(LPARAM)WSTAT_MUSVOL_PAGE_SIZE);
	SendMessage(mhTrack_musVol, TBM_SETTICFREQ,
		(WPARAM)WSTAT_MUSVOL_PAGE_SIZE,
		(LPARAM)0);
	SendMessage(mhTrack_musVol, TBM_SETPOS,
		(WPARAM)TRUE,
		(LPARAM)CORETONE_DEFAULT_VOLUME);
	dwLast_musVol = CORETONE_DEFAULT_VOLUME;


	/**
	 * ---- SOUND EFFECT MANAGER ----
	 */
	miActiveSfx = 0;
	mhIconPlay_A = (HICON)LoadImage(
		GetModuleHandle(NULL),
		MAKEINTRESOURCE(IDI_PLAY_ACTIVE),
		IMAGE_ICON,
		16, 16,
		LR_DEFAULTCOLOR);
	mhIconPlay_G = (HICON)LoadImage(
		GetModuleHandle(NULL),
		MAKEINTRESOURCE(IDI_PLAY_INACTIVE),
		IMAGE_ICON,
		16, 16,
		LR_DEFAULTCOLOR);

	mhIconStop_A = (HICON)LoadImage(
		GetModuleHandle(NULL),
		MAKEINTRESOURCE(IDI_STOP_ACTIVE),
		IMAGE_ICON,
		16, 16,
		LR_DEFAULTCOLOR);
	mhIconStop_G = (HICON)LoadImage(
		GetModuleHandle(NULL),
		MAKEINTRESOURCE(IDI_STOP_INACTIVE),
		IMAGE_ICON,
		16, 16,
		LR_DEFAULTCOLOR);

	midBorder_sfxSel = WSTAT_SFXSELB_ID;
	mhBorder_sfxSel = CreateWindowEx(
		(0),									/* dwExStyle */
		WC_STATIC,								/* lpClassName */
		szName_sfxSelB,							/* lpWindowName */
		(WS_CHILD | WS_VISIBLE)
			| (SS_ETCHEDFRAME | SS_LEFT),		/* dwStyle */
		WSTAT_SFXSELB_X,						/* x */
		WSTAT_SFXSELB_Y,						/* y */
		WSTAT_SFXSELB_WIDTH,					/* nWidth */
		WSTAT_SFXSELB_HEIGHT,					/* nHeight */
		mhParent,								/* hWndParent */
		(HMENU)midBorder_sfxSel,				/* idControl */
		GetModuleHandle(NULL),					/* hInstance */
		NULL);									/* lpParam */
	SetWindowText(mhBorder_sfxSel, szName_ctEmpty);

	midLabel_sfxSel = WSTAT_SFXSELL_ID;
	mhLabel_sfxSel = CreateWindowEx(
		(0),									/* dwExStyle */
		WC_STATIC,								/* lpClassName */
		szName_sfxSelL,							/* lpWindowName */
		(WS_CHILD | WS_VISIBLE)
			| (SS_LEFT),						/* dwStyle */
		WSTAT_SFXSELL_X,						/* x */
		WSTAT_SFXSELL_Y,						/* y */
		WSTAT_SFXSELL_WIDTH,					/* nWidth */
		WSTAT_SFXSELL_HEIGHT,					/* nHeight */
		mhParent,								/* hWndParent */
		(HMENU)midLabel_sfxSel,					/* idControl */
		GetModuleHandle(NULL),					/* hInstance */
		NULL);									/* lpParam */

	midPlay_sfxSel = WSTAT_SFXSELP_ID;
	mhPlay_sfxSel = CreateWindowEx(
		(0),									/* dwExStyle */
		WC_BUTTON,								/* lpClassName */
		szName_sfxSelP,							/* lpWindowName */
		(WS_CHILD | WS_VISIBLE)
			| (BS_ICON | BS_PUSHBUTTON),		/* dwStyle */
		WSTAT_SFXSELP_X,						/* x */
		WSTAT_SFXSELP_Y,						/* y */
		WSTAT_SFXSELP_WIDTH,					/* nWidth */
		WSTAT_SFXSELP_HEIGHT,					/* nHeight */
		mhParent,								/* hWndParent */
		(HMENU)midPlay_sfxSel,					/* idControl */
		GetModuleHandle(NULL),					/* hInstance */
		NULL);									/* lpParam */
	SendMessage(mhPlay_sfxSel, BM_SETIMAGE,
		(WPARAM)IMAGE_ICON,
		(LPARAM)mhIconPlay_G);

	midStop_sfxSel = WSTAT_SFXSELS_ID;
	mhStop_sfxSel = CreateWindowEx(
		(0),									/* dwExStyle */
		WC_BUTTON,								/* lpClassName */
		szName_sfxSelS,							/* lpWindowName */
		(WS_CHILD | WS_VISIBLE)
			| (BS_ICON | BS_PUSHBUTTON),		/* dwStyle */
		WSTAT_SFXSELS_X,						/* x */
		WSTAT_SFXSELS_Y,						/* y */
		WSTAT_SFXSELS_WIDTH,					/* nWidth */
		WSTAT_SFXSELS_HEIGHT,					/* nHeight */
		mhParent,								/* hWndParent */
		(HMENU)midStop_sfxSel,					/* idControl */
		GetModuleHandle(NULL),					/* hInstance */
		NULL);									/* lpParam */
	SendMessage(mhStop_sfxSel, BM_SETIMAGE,
		(WPARAM)IMAGE_ICON,
		(LPARAM)mhIconStop_G);


	midTrack_sfxPri = WSTAT_SFXPRI_ID;
	mhTrack_sfxPri = CreateWindowEx(
		(0),									/* dwExStyle */
		TRACKBAR_CLASS,							/* lpClassName */
		szName_sfxPriT,							/* lpWindowName */
		(WS_CHILD | WS_VISIBLE)
			| (TBS_AUTOTICKS),					/* dwStyle */
		WSTAT_SFXPRI_X,							/* x */
		WSTAT_SFXPRI_Y,							/* y */
		WSTAT_SFXPRI_WIDTH,						/* nWidth */
		WSTAT_SFXPRI_HEIGHT,					/* nHeight */
		mhParent,								/* hWndParent */
		(HMENU)midTrack_sfxPri,					/* idControl */
		GetModuleHandle(NULL),					/* hInstance */
		NULL);									/* lpParam */
	SendMessage(mhTrack_sfxPri, TBM_SETRANGE,
		(WPARAM)TRUE,
		(LPARAM)MAKELONG(WSTAT_SFXPRI_MIN, WSTAT_SFXPRI_MAX));
	SendMessage(mhTrack_sfxPri, TBM_SETPAGESIZE,
		0,
		(LPARAM)WSTAT_SFXPRI_PAGE_SIZE);
	SendMessage(mhTrack_sfxPri, TBM_SETTICFREQ,
		(WPARAM)WSTAT_SFXPRI_PAGE_SIZE,
		(LPARAM)0);
	SendMessage(mhTrack_sfxPri, TBM_SETPOS,
		(WPARAM)TRUE,
		(LPARAM)127);
	miPlayPri = 127;

	midTrack_sfxPan = WSTAT_SFXPAN_ID;
	mhTrack_sfxPan = CreateWindowEx(
		(0),									/* dwExStyle */
		TRACKBAR_CLASS,							/* lpClassName */
		szName_sfxPanT,							/* lpWindowName */
		(WS_CHILD | WS_VISIBLE)
			| (TBS_AUTOTICKS),					/* dwStyle */
		WSTAT_SFXPAN_X,							/* x */
		WSTAT_SFXPAN_Y,							/* y */
		WSTAT_SFXPAN_WIDTH,						/* nWidth */
		WSTAT_SFXPAN_HEIGHT,					/* nHeight */
		mhParent,								/* hWndParent */
		(HMENU)midTrack_sfxPan,					/* idControl */
		GetModuleHandle(NULL),					/* hInstance */
		NULL);									/* lpParam */
	SendMessage(mhTrack_sfxPan, TBM_SETRANGE,
		(WPARAM)TRUE,
		(LPARAM)MAKELONG(WSTAT_SFXPAN_MIN, WSTAT_SFXPAN_MAX));
	SendMessage(mhTrack_sfxPan, TBM_SETPAGESIZE,
		0,
		(LPARAM)WSTAT_SFXPAN_PAGE_SIZE);
	SendMessage(mhTrack_sfxPan, TBM_SETTICFREQ,
		(WPARAM)WSTAT_SFXPAN_PAGE_SIZE,
		(LPARAM)0);
	SendMessage(mhTrack_sfxPan, TBM_SETPOS,
		(WPARAM)TRUE,
		(LPARAM)128);
	miPlayPan = 128;
	miVol_Left = 127;
	miVol_Right = 127;
}

StatMan::~StatMan(void)
{
	if(NULL != mhWaveMap)
	{
		DeleteObject(mhWaveMap);
		mhWaveMap = NULL;
	}
	if(NULL != mhDCm)
	{
		DeleteDC(mhDCm);
		mhDCm = NULL;
	}

	if(NULL != mhIconPlay_A)
	{
		DestroyIcon(mhIconPlay_A);
		mhIconPlay_A = NULL;
	}
	if(NULL != mhIconPlay_G)
	{
		DestroyIcon(mhIconPlay_G);
		mhIconPlay_G = NULL;
	}

	if(NULL != mhIconStop_A)
	{
		DestroyIcon(mhIconStop_A);
		mhIconStop_A = NULL;
	}
	if(NULL != mhIconStop_G)
	{
		DestroyIcon(mhIconStop_G);
		mhIconStop_G = NULL;
	}
}




/******************************************************************************
 * !!!!----                   WINPROC/UI HANDLERS                    ----!!!!
 ******************************************************************************/
/* LRESULT StatMan::CmdProc(HWND hWnd, UINT uMsg,
 *                          WPARAM wParam, LPARAM lParam)
 * WM_COMMAND handler for the sound effect manager buttons, this also traps
 * the menu and accelerators.
 *----------------------------------------------------------------------------*/
LRESULT StatMan::CmdProc(HWND hWnd, UINT uMsg,
                         WPARAM wParam, LPARAM lParam)
{
	switch(LOWORD(wParam))
	{
		case IDM_PLAYBACK_SFX_PLAY:
		case IDB_SFX_PLAY:
			if(mpToneMan->mbDriverRunning && mpToneMan->mbSfxReady)
			{
				mpToneMan->PlaySfx(miPlayPri, miVol_Left, miVol_Right);
			}
			break;

		case IDM_PLAYBACK_SFX_STOP:
		case IDB_SFX_STOP:
			if(mpToneMan->mbDriverRunning && mpToneMan->mbSfxReady)
			{
				mpToneMan->StopSfx();
			}
			break;
	}


	if(!miActiveSfx && mpToneMan->mbSfxReady && mpToneMan->mbDriverRunning)
	{
		SendMessage(mhPlay_sfxSel, BM_SETIMAGE,
			(WPARAM)IMAGE_ICON,
			(LPARAM)mhIconPlay_A);
		SendMessage(mhStop_sfxSel, BM_SETIMAGE,
			(WPARAM)IMAGE_ICON,
			(LPARAM)mhIconStop_A);

		miActiveSfx = -1;
	}

	return 0;
}

/* LRESULT StatMan::TrackProc(HWND hWnd, UINT uMsg,
 *                            WPARAM wParam, LPARAM lParam)
 * WM_HSCROLL handler for the PlayTone window callback, specifically trackbars.
 *----------------------------------------------------------------------------*/
LRESULT StatMan::TrackProc(HWND hWnd, UINT uMsg,
                           WPARAM wParam, LPARAM lParam)
{
	DWORD dwPos,dwPosMus,dwPosPri,dwPosPan;
	INT iPri,iPan;
	BYTE cVol;

	dwPos = SendMessage(mhTrack_musVol, TBM_GETPOS, 0, 0);
	dwPos = (dwPos > WSTAT_MUSVOL_MAX)
		? WSTAT_MUSVOL_MAX
		: ((dwPos < WSTAT_MUSVOL_MIN) ? WSTAT_MUSVOL_MIN : dwPos);
	dwPosMus = dwPos;
	cVol = (BYTE)dwPos;

	dwPos = SendMessage(mhTrack_sfxPri, TBM_GETPOS, 0, 0);
	dwPos = (dwPos > WSTAT_SFXPRI_MAX)
		? WSTAT_SFXPRI_MAX
		: ((dwPos < WSTAT_SFXPRI_MIN) ? WSTAT_SFXPRI_MIN : dwPos);
	dwPosPri = dwPos;
	iPri = (INT)dwPos;

	dwPos = SendMessage(mhTrack_sfxPan, TBM_GETPOS, 0, 0);
	dwPos = (dwPos > WSTAT_SFXPAN_MAX)
		? WSTAT_SFXPAN_MAX
		: ((dwPos < WSTAT_SFXPAN_MIN) ? WSTAT_SFXPAN_MIN : dwPos);
	dwPosPan = dwPos;
	iPan = (INT)dwPos;

	/**
	 * Control over the music volume slider is allowed as long as the
	 * driver is running, however the sound effect manager can only be
	 * manipulated once a sound effect has been loaded.
	 */
	if(mpToneMan->mbDriverRunning)
	{
		mpToneMan->AttenMusic(cVol);
		dwLast_musVol = dwPosMus;
	}
	else
	{
		dwPosMus = dwLast_musVol;
	}

	if(mpToneMan->mbDriverRunning && mpToneMan->mbSfxReady)
	{
		if(iPri != miPlayPri)
		{
			mpToneMan->StopSfx();
		}
		miPlayPri = iPri;
		miPlayPan = iPan;
		miVol_Left = (iPan >= 128) ? (127 - (iPan - 128)) : 127;
		miVol_Right = (iPan < 128) ? iPan : 127;
	}
	else
	{
		dwPosPri = miPlayPri;
		dwPosPan = miPlayPan;
	}

	SendMessage(mhTrack_musVol,
		TBM_SETPOS,
		(WPARAM)TRUE, (LPARAM)dwPosMus);
	SendMessage(mhTrack_sfxPri,
		TBM_SETPOS,
		(WPARAM)TRUE, (LPARAM)dwPosPri);
	SendMessage(mhTrack_sfxPan,
		TBM_SETPOS,
		(WPARAM)TRUE, (LPARAM)dwPosPan);
	return 0;
}


/* LRESULT StatMan::PaintProc(HWND hWnd, HDC hDC, PAINTSTRUCT psWin)
 *----------------------------------------------------------------------------*/
LRESULT StatMan::PaintProc(HWND hWnd, HDC hDC, PAINTSTRUCT psWin)
{
	HFONT hFontNew,hFontPrv;
	HBRUSH hBrushFill,hBrushPrev;
	HBITMAP hPrevMap;

	COLORREF crWindow;
	COLORREF crLeft,crRight,crBoth;

	RECT rText;
	INT iMood;
	UINT uiTicks;
	double dTicks,dRate,dSec;

	/**
	 * Set up the colors we're using to draw text and the wave view
	 * before proceeding... Note that the value we get from PALETTERGB
	 * is not necessarily the draw value due to varying color depths.
	 */
	crWindow = GetSysColor(COLOR_3DFACE);
	crLeft = PALETTERGB(0xFF,0x00,0x00);
	crRight = PALETTERGB(0x00,0x00,0xFF);
	crBoth = PALETTERGB(0xFF,0x00,0xFF);

	/**
	 * ---- Text Draw ----
	 */
	SetBkMode(hDC, OPAQUE);
	SetBkColor(hDC, crWindow);
	hFontNew = (HFONT)GetStockObject(ANSI_VAR_FONT);

	hFontPrv = (HFONT)SelectObject(hDC, hFontNew);
	if(NULL != hFontPrv)
	{
		wt_getState(NULL, NULL, NULL, NULL, &uiTicks,
			NULL, NULL);
		dTicks = uiTicks;
		dRate = muiDecodeRate;
		dSec = dTicks / dRate;

		iMood = wt_getMood();

		sprintf(mszRender, szData_ctInfoS,
			muiCoreChannels,
			muiRenderFreq, muiDecodeRate,
			muiSamples, muiSampleLen,
			iMood,
			(mpToneMan->mbDriverRunning ? szData_ctInfoR : szData_ctInfoI),
			uiTicks, dSec);
		rText.top = (WSTAT_CTINFO_Y + 2);
		rText.left = (WSTAT_CTINFO_X + 0);
		rText.right = (WSTAT_CTINFO_Y + WSTAT_CTINFO_WIDTH);
		rText.bottom = (WSTAT_CTINFO_X + WSTAT_CTINFO_HEIGHT + 3);
		DrawText(hDC, mszRender, -1, &rText,
			(DT_LEFT | DT_WORDBREAK | DT_EXTERNALLEADING));

		TextOut(hDC, WSTAT_MUSVOL_SOFT_X, WSTAT_MUSVOL_SOFT_Y,
			szLabel_musVolSoft,
			(sizeof(szLabel_musVolSoft) / sizeof(szLabel_musVolSoft[0])) - 1);
		TextOut(hDC, WSTAT_MUSVOL_LOUD_X, WSTAT_MUSVOL_LOUD_Y,
			szLabel_musVolLoud,
			(sizeof(szLabel_musVolLoud) / sizeof(szLabel_musVolLoud[0])) - 1);

		TextOut(hDC, WSTAT_SFXPRI_CAT_X, WSTAT_SFXPRI_CAT_Y,
			szCat_sfxPri,
			(sizeof(szCat_sfxPri) / sizeof(szCat_sfxPri[0])) - 1);
		TextOut(hDC, WSTAT_SFXPAN_CAT_X, WSTAT_SFXPAN_CAT_Y,
			szCat_sfxPan,
			(sizeof(szCat_sfxPan) / sizeof(szCat_sfxPan[0])) - 1);

		TextOut(hDC, WSTAT_SFXPRI_LABEL_X, WSTAT_SFXPRI_LABEL_Y,
			szLabel_sfxPri,
			(sizeof(szLabel_sfxPri) / sizeof(szLabel_sfxPri[0])) - 1);
		TextOut(hDC, WSTAT_SFXPAN_LABEL_X, WSTAT_SFXPAN_LABEL_Y,
			szLabel_sfxPan,
			(sizeof(szLabel_sfxPan) / sizeof(szLabel_sfxPan[0])) - 1);

		SelectObject(hDC, hFontPrv);
	}

	/**
	 * ---- WAVE VUE ----
	 */
	if(NULL == mhDCm)
	{
		mhDCm = CreateCompatibleDC(hDC);
	}
	if(NULL != mhDCm)
	{
		if(NULL == mhWaveMap)
		{
			mhWaveMap = CreateCompatibleBitmap(hDC,
				WSTAT_WAVEVU_WIDTH, WSTAT_WAVEVU_HEIGHT);
		}

		if(NULL != mhWaveMap)
		{
			/**
			 * Alright, we've got our display compatible bitmap ready to go.
			 * We're going to render our wave in four passes:
			 *
			 * 1) Force our COLORREFs to match the pixel properties of
			 *    the destination bitmap by doing a Set(),Get() on each.
			 * 2) Clear the bitmap to the window color.
			 * 3,4) Draw the LEFT & RIGHT channel waveforms.
			 */
			hPrevMap = (HBITMAP)SelectObject(mhDCm, mhWaveMap);
			crLeft = SetPixel(mhDCm, 0, 0, crLeft), GetPixel(mhDCm, 0, 0);
			crRight = SetPixel(mhDCm, 0, 0, crRight), GetPixel(mhDCm, 0, 0);
			crBoth = SetPixel(mhDCm, 0, 0, crBoth), GetPixel(mhDCm, 0, 0);

			hBrushFill = CreateSolidBrush(crWindow);
			hBrushPrev = (HBRUSH)SelectObject(mhDCm, hBrushFill);
			Rectangle(mhDCm, -1, -1,
				(WSTAT_WAVEVU_WIDTH + 1), (WSTAT_WAVEVU_HEIGHT + 1));
			SelectObject(mhDCm, hBrushPrev);
			DeleteObject(hBrushFill);

			RenderWave(mhDCm, crLeft, crRight, crBoth);

			/**
			 * Bit the Blits, er... Blit the Bits!
			 */
			BitBlt(hDC,
				WSTAT_WAVEVU_X, WSTAT_WAVEVU_Y,
				WSTAT_WAVEVU_WIDTH, WSTAT_WAVEVU_HEIGHT,
				mhDCm, 0, 0,
				SRCCOPY);

			SelectObject(mhDCm, hPrevMap);
		}
	}
	return 0;
}


/* LRESULT StatMan::InvalProc(HWND hWnd)
 *----------------------------------------------------------------------------*/
LRESULT StatMan::InvalProc(HWND hWnd)
{
	RECT rInfo,rTrack,rMan,rVue;

	rInfo.left = WSTAT_CTINFO_X;
	rInfo.right = (WSTAT_CTINFO_X + WSTAT_CTINFO_WIDTH);
	rInfo.top = WSTAT_CTINFO_Y;
	rInfo.bottom = (WSTAT_CTINFO_Y + WSTAT_CTINFO_HEIGHT + 5);

	rTrack.left = WSTAT_MUSVOL_X;
	rTrack.right = (WSTAT_MUSVOL_X + WSTAT_MUSVOL_WIDTH);
	rTrack.top = WSTAT_MUSVOL_SOFT_Y;
	rTrack.bottom = (WSTAT_MUSVOL_SOFT_Y + 16);

	rMan.left = WSTAT_SFXSEL_X;
	rMan.right = (WSTAT_SFXSEL_X + WSTAT_SFXSEL_WIDTH);
	rMan.top = WSTAT_SFXSEL_Y;
	rMan.bottom = (WSTAT_SFXSEL_Y + WSTAT_SFXSEL_HEIGHT);
	
	rVue.left = WSTAT_WAVEVU_X;
	rVue.right = (WSTAT_WAVEVU_X + WSTAT_WAVEVU_WIDTH);
	rVue.top = WSTAT_WAVEVU_Y;
	rVue.bottom = (WSTAT_WAVEVU_Y + WSTAT_WAVEVU_HEIGHT);

	InvalidateRect(hWnd, &rInfo, FALSE);
	InvalidateRect(hWnd, &rTrack, FALSE);
	InvalidateRect(hWnd, &rMan, FALSE);
	InvalidateRect(hWnd, &rVue, FALSE);
	return 0;
}


/* void StatMan::RenderWave(HDC hDCm, COLORREF crLeft, COLORREF crRight,
 *                          COLORREF crMerge)
 * Render the CoreTone stereo waveform to hDCm, which should have the
 * dimensions (WSTAT_CTINFO_WIDTH, WSTAT_CTINFO_HEIGHT).
 *----------------------------------------------------------------------------*/
void StatMan::RenderWave(HDC hDCm, COLORREF crLeft, COLORREF crRight,
						 COLORREF crMerge)
{
	double dSrcW,dDestW;
	double dAdj,dAdj_int,dAdj_frac;
	int32p32_t lSrc,lAdj;

	int16_t sSample;
	int16_t *psWave;
	UINT uiWaveLen,uiChanLen;

	UINT uiVertAdj,uiCenter;
	UINT uiX,uiY;

	wt_getState(NULL, NULL, NULL, NULL, NULL, &psWave, &uiWaveLen);
	uiChanLen = uiWaveLen >> 1;

	uiVertAdj = 65536 / WSTAT_CTINFO_HEIGHT;
	uiCenter = WSTAT_CTINFO_HEIGHT / 2;

	dSrcW = uiChanLen;
	dDestW = WSTAT_CTINFO_WIDTH;
	dAdj = dSrcW / dDestW;

	dAdj_int = floor(dAdj);
	dAdj_frac = dAdj - dAdj_int;
	dAdj_frac *= 4294967296.0;
	lAdj.uiPair.uiHi = (uint32_t)dAdj_int;
	lAdj.uiPair.uiLo = (uint32_t)dAdj_frac;

	uiX = 0;
	lSrc.lWhole = 0;
	while((uiX < WSTAT_CTINFO_WIDTH) && (lSrc.uiPair.uiHi < uiChanLen))
	{
		/**
		 * LEFT Channel
		 */
		sSample = -1 * psWave[(lSrc.uiPair.uiHi * 2) + 0];
		uiY = sSample;
		uiY = (uiY + 32768) / uiVertAdj;

		if(uiY >= uiCenter)
		{
			while(uiY >= uiCenter)
			{
				SetPixel(hDCm, uiX, uiY, crLeft);
				uiY--;
			}
		}
		else
		{
			while(uiY < uiCenter)
			{
				SetPixel(hDCm, uiX, uiY, crLeft);
				uiY++;
			}
		}

		/**
		 * RIGHT Channel
		 */
		sSample = -1 * psWave[(lSrc.uiPair.uiHi * 2) + 1];
		uiY = sSample;
		uiY = (uiY + 32768) / uiVertAdj;

		if(uiY >= uiCenter)
		{
			while(uiY >= uiCenter)
			{
				SetPixel(hDCm, uiX, uiY,
					((GetPixel(hDCm, uiX, uiY) == crLeft)
						? crMerge
						: crRight));
				uiY--;
			}
		}
		else
		{
			while(uiY < uiCenter)
			{
				SetPixel(hDCm, uiX, uiY,
					((GetPixel(hDCm, uiX, uiY) == crLeft)
						? crMerge
						: crRight));
				uiY++;
			}
		}

		lSrc.lWhole += lAdj.lWhole;
		uiX++;
	}
}
